@@ -33,22 +33,14 @@ class View;
 
 namespace Ember
 {
-namespace EntityMapping
+namespace Domain
 {
-class EntityMapping;
-}
-}
-
-namespace Ember
-{
-namespace Domain {
 class EntityTalk;
+class IHeightProvider;
 }
 namespace OgreView
 {
 
-class Scene;
-
 namespace Model
 {
 class Model;
@@ -110,6 +102,18 @@ public:
 		PM_SWIMMING = 4
 	};
 
+	/**
+	 * @brief The different composition modes available.
+	 * Some entities can be "composition" entities. That means that they and their children make up one single conceptual entity.
+	 * An example might be a table, where each table leg is a separate entity. Each leg is then part of the "table composition".
+	 */
+	enum CompositionMode
+	{
+		CM_DISABLED, //!< CM_DISABLED No composition is used. This should be the default.
+		CM_COMPOSITION, //!< CM_COMPOSITION The entity is an composite. That means that when interacting with a child entity the user should also be given the option to interact with the parent composite entity.
+		CM_COMPOSITION_EXCLUSIVE //!< CM_COMPOSITION_EXCLUSIVE The entity is an exclusive composite. That means that when interacting with a child entity the user should only be presented with the parent composite entity (and should not be able to interact with the child entity).
+	};
+
 	/**
 	 * @brief The name of the "floating" positioning mode.
 	 */
@@ -136,21 +140,14 @@ public:
 	 * @param id The unique id of the entity.
 	 * @param ty The type info which specified the type for this entity.
 	 * @param vw The world view to which this entity belongs.
-	 * @param scene The scene to which this entity belongs.
 	 */
-	EmberEntity(const std::string& id, Eris::TypeInfo* ty, Eris::View* vw, Scene& scene);
+	EmberEntity(const std::string& id, Eris::TypeInfo* ty, Eris::View* vw);
 
 	/**
 	 * @brief Dtor.
 	 */
 	virtual ~EmberEntity();
 
-	/**
-	 * @brief Creates the model mapping for this entity. Call this once when initializing the entity.
-	 * @param scene The scene to which any created model should belong.
-	 */
-	void createEntityMapping(Scene& scene);
-
 	/**
 	 * @brief Adjust the height of the entity so that it "snaps" to the ground or is otherwise adjusted, depending on the current movement mode.
 	 * This is most often done by making a call to the containing node's adjustPositionForContainedNode method.
@@ -226,14 +223,14 @@ public:
 
 	/**
 	 * General method for turning on and off debug visualizations. Subclasses might support more types of visualizations than the ones defined here.
-	 * @param visualization The type of visualization. Currently supports "OgreBBox" and "ErisBBox".
+	 * @param visualization The type of visualization. Currently supports "OgreBBox".
 	 * @param visualize Whether to visualize or not.
 	 */
 	void setVisualize(const std::string& visualization, bool visualize);
 
 	/**
 	 * @brief Gets whether a certain visualization is turned on or off.
-	 * @param visualization The type of visualization. Currently supports "OgreBBox" and "ErisBBox".
+	 * @param visualization The type of visualization. Currently supports "OgreBBox".
 	 * @return true if visualization is turned on, else false
 	 */
 	bool getVisualize(const std::string& visualization) const;
@@ -305,13 +302,6 @@ public:
 	 */
 	void accept(IEntityVisitor& visitor);
 
-	/**
-	 * @brief Accessor for the mapping attached to this entity.
-	 * @returns A mapping instance, or null if one hasn't been set up yet.
-	 */
-	EntityMapping::EntityMapping* getMapping() const;
-
-
 	/**
 	 * @brief Gets the height at the local position.
 	 * @param localPosition A position local to the entity.
@@ -319,6 +309,11 @@ public:
 	 */
 	virtual float getHeight(const WFMath::Point<2>& localPosition) const;
 
+	void setHeightProvider(Domain::IHeightProvider* heightProvider);
+
+	CompositionMode getCompositionMode() const;
+	void setCompositionMode(CompositionMode mode);
+
 	sigc::signal<void, const Domain::EntityTalk&> EventTalk;
 
 protected:
@@ -334,34 +329,36 @@ protected:
 	 */
 	std::vector<std::string> mSuggestedResponses;
 
-	/**
-	 * @brief If there's a terrain area belonging to this entity, that's stored here.
-	 * The terrain area instance will take care of all required terrain area functionality once it's been created, offloading this from the EmberEntity. Most entities won't however have any terrain areas, for which this will be null.
-	 */
-	std::auto_ptr<Terrain::TerrainArea> mTerrainArea;
-
-	/**
-	 * @brief If a terrainmod belongs to this entity, it's stored here.
-	 * The terrain mod instance will take care of all required terrain mod functionality once it's been created, offloading this from the EmberEntity. Most entities won't however have any terrain mods, for which this will be null.
-	 */
-	std::auto_ptr<Terrain::TerrainMod> mTerrainMod;
+//	/**
+//	 * @brief If there's a terrain area belonging to this entity, that's stored here.
+//	 * The terrain area instance will take care of all required terrain area functionality once it's been created, offloading this from the EmberEntity. Most entities won't however have any terrain areas, for which this will be null.
+//	 */
+//	std::unique_ptr<Terrain::TerrainArea> mTerrainArea;
+//
+//	/**
+//	 * @brief If a terrainmod belongs to this entity, it's stored here.
+//	 * The terrain mod instance will take care of all required terrain mod functionality once it's been created, offloading this from the EmberEntity. Most entities won't however have any terrain mods, for which this will be null.
+//	 */
+//	std::unique_ptr<Terrain::TerrainMod> mTerrainMod;
 
 	/**
 	 * @brief The positioning mode the entity is in, like gravity affected, fixed or floating.
 	 */
 	PositioningMode mPositioningMode;
 
+	/**
+	 * @brief The composition mode for the entity.
+	 *
+	 * Is "none" (CM_DISABLED) by default.
+	 */
+	CompositionMode mCompositionMode;
+
 	/**
 	 * @brief The graphical representation used for representing this entity.
 	 * Some entities won't have any graphical representation, and this will in those cases be null.
 	 */
 	IGraphicalRepresentation* mGraphicalRepresentation;
 
-	/**
-	 * @brief The entity mapping used for this entity.
-	 */
-	EntityMapping::EntityMapping* mEntityMapping;
-
 	/**
 	 * @brief The attachment for this entity.
 	 *
@@ -376,6 +373,11 @@ protected:
 	 */
 	IEntityControlDelegate* mAttachmentControlDelegate;
 
+	/**
+	 * @brief An optional height provider attached to this entity.
+	 */
+	Domain::IHeightProvider* mHeightProvider;
+
 	/**
 	 *    @copydoc Eris::Entity::onTalk()
 	 */
@@ -388,10 +390,6 @@ protected:
 	 *    @copydoc Eris::Entity::onAction()
 	 */
 	virtual void onAction(const Atlas::Objects::Operation::RootOperation& act);
-	/**
-	 *    @copydoc Eris::Entity::onImaginary()
-	 */
-	virtual void onImaginary(const Atlas::Objects::Root& act);
 	/**
 	 *    @copydoc Eris::Entity::onSoundAction()
 	 */
@@ -402,8 +400,6 @@ protected:
 	 */
 	virtual void onAttrChanged(const std::string& str, const Atlas::Message::Element& v);
 
-
-
 	/**
 	 * @brief Called when the positioning mode of the entity changes.
 	 * Depending on the kind of positioning mode the entity needs to be adjusted in the world accordinly. For example, when the entity is gravity affected (the default) it should snap to the terrain, in contrast to when the mode is "fixed" where it wouldn't snap to the terrain.
@@ -416,38 +412,6 @@ protected:
 	 */
 	virtual void onBboxChanged();
 
-	/**
-	 * @brief Adds a terrain area to the entity.
-	 * Areas are a terrain feature, and cannot be handled by a standard implementation of EmberEntity. The default implementation of this method in this class will therefore only pass it on to the parent entity, if any such is available. The idea is that somewhere along the way, as the method "walks" the entity hierarcy upwards it will come across an implementation of this method that does indeed know how to handle the terrain area.
-	 * @see OgreView::WorldEmberEntity::addArea()
-	 * @param area The area which has been added.
-	 */
-	virtual void addArea(Terrain::TerrainArea& area);
-
-	/**
-	 * @brief Adds a terrain mod to the entity.
-	 * Mods are a terrain feature, and cannot be handled by a standard implementation of EmberEntity. The default implementation of this method in this class will therefore only pass it on to the parent entity, if any such is available. The idea is that somewhere along the way, as the method "walks" the entity hierarcy upwards it will come across an implementation of this method that does indeed know how to handle the terrain mod.
-	 * @see OgreView::WorldEmberEntity::addTerrainMod()
-	 * @param mod The mod which has been added.
-	 */
-	virtual void addTerrainMod(Terrain::TerrainMod* mod);
-
-	/**
-	 * @brief Updates the terrain with the specified terrain definition points.
-	 * @param terrainDefinitionPoints The terrain definition points.
-	 */
-	virtual void updateTerrain(const std::vector<Terrain::TerrainDefPoint>& terrainDefinitionPoints);
-
-	/**
-	 * @brief Checks if we should create any dependent objects, such as TerrainArea or TerrainMod.
-	 * Call this only when the entity has been propely initialized, since we then have all the correct data regarding positions, bounding boxes etc.
-	 * @see init()
-	 * @see onAttrChanged()
-	 * @param attributeName The attribute which we should try to create something for.
-	 * @return True if any dependent object was created.
-	 */
-	virtual bool createDependentObject(const std::string& attributeName);
-
 	/**
 	 * @brief Called by eris just after the entity has been put into the world.
 	 * @param ge The root entity which contains all atlas data that define this entity.
@@ -469,18 +433,6 @@ protected:
 	 */
 	void reattachChildren();
 
-	/**
-	 * @brief Shows the eris/atlas bounding box.
-	 * This is mainly useful for debugging or authoring.
-	 * @param show If true, the bounding box will be shown. If false, it will be hidden.
-	 */
-	void showErisBoundingBox(bool show);
-
-	/**
-	 * @brief Returns whether the eris/atlas bounding box is shown.
-	 * @return True if the eris bounding box is shown.
-	 */
-	bool getShowErisBoundingBox() const;
 };
 
 inline bool EmberEntity::isInitialized() const
