@@ -77,7 +77,7 @@ namespace Caelum
     {
         if (!CaelumSystemTypeDescriptor)
         {
-            std::auto_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
+            std::unique_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
 
             // Timing settings.
             td->add("time_scale",
@@ -152,7 +152,7 @@ namespace Caelum
 
         if (!PointStarfieldTypeDescriptor)
         {
-            std::auto_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
+            std::unique_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
             td->add("magnitude_scale",
                     new AccesorPropertyDescriptor<Caelum::PointStarfield, Real, Real, Real>(
                             &Caelum::PointStarfield::getMagnitudeScale,
@@ -186,7 +186,7 @@ namespace Caelum
 
         if (!BaseSkyLightTypeDescriptor)
         {
-            std::auto_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
+            std::unique_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
             td->add("ambient_multiplier",
                     new AccesorPropertyDescriptor<Caelum::BaseSkyLight, ColourValue>(
                             &Caelum::BaseSkyLight::getAmbientMultiplier,
@@ -220,7 +220,7 @@ namespace Caelum
 
         if (!GroundFogTypeDescriptor)
         {
-            std::auto_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
+            std::unique_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
             td->add("density",
                     new AccesorPropertyDescriptor<Caelum::GroundFog, Real, Real, Real>(
                             &Caelum::GroundFog::getDensity,
@@ -242,7 +242,7 @@ namespace Caelum
 
         if (!DepthComposerTypeDescriptor)
         {
-            std::auto_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
+            std::unique_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
             td->add("debug_depth_render",
                     new AccesorPropertyDescriptor<Caelum::DepthComposer, bool, bool, bool>(
                             &Caelum::DepthComposer::getDebugDepthRender,
@@ -289,7 +289,7 @@ namespace Caelum
 
         if (!PrecipitationTypeDescriptor)
         {
-            std::auto_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
+            std::unique_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
 
             td->add("texture",
                     new AccesorPropertyDescriptor<Caelum::PrecipitationController, String>(
@@ -329,7 +329,7 @@ namespace Caelum
 
         if (!FlatCloudLayerTypeDescriptor)
         {
-            std::auto_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
+            std::unique_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
 
             // Height.
             td->add("height",
@@ -428,7 +428,7 @@ namespace Caelum
 
         if (!SkyDomeTypeDescriptor)
         {
-            std::auto_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
+            std::unique_ptr<DefaultTypeDescriptor> td (new DefaultTypeDescriptor ());
 
             // SkyDome is slightly special because most properties are write-only.
 
