@@ -1,7 +1,6 @@
 #include "TStem.h"
 #include "MeshTree.h"
-#include "framework/float_cast.h"
-
+#include <cmath>
 //---------------------------------------------------------------------------
 
 /*
@@ -18,20 +17,16 @@
 
 */
 
-#ifdef _MSC_VER
-#define lrintf (int)
-#endif
-
 namespace Ogre {
 
 inline int Round(const Real fValue)
 {
-  return fValue<0 ?  lrintf(fValue-0.5) : lrintf(fValue+0.5);
+  return fValue<0 ?  (int)(fValue-0.5) : (int)(fValue+0.5);
 }
 
 inline int Trunc(const Real fValue)
 {
-  return lrintf(fValue);
+  return (int)(fValue);
 }
 
 //===========================================================================
@@ -915,8 +910,8 @@ void TStem::AddLeavesVertices(Real **pVertexArray, RGBA **pVertexColorArray, con
 	  pLeaf = mVectorOfLeaves[i];
 	  u16NbVertices = (uint)pLeaf->size();
       iLeafType = Round( mpTree->GetRandomValue(3.99) );
-      fTexCoordOffsetU = 0.5 * (iLeafType / 2);
-      fTexCoordOffsetV = 0.5 * (iLeafType % 2);
+      fTexCoordOffsetU = 0.5 * (Ogre::Real)(iLeafType / 2);
+      fTexCoordOffsetV = 0.5 * (Ogre::Real)(iLeafType % 2);
 
       for (j=0; j<u16NbVertices; j++)
 	  {
