@@ -209,6 +209,8 @@ public:
 		heightFunctionUserData = userData;
 	}
 
+	bool preparePage(PageInfo &page);
+
 	/** INTERNAL FUNCTION - DO NOT USE */
 	void loadPage(PageInfo &page);
 	/** INTERNAL FUNCTION - DO NOT USE */
@@ -386,13 +388,14 @@ public:
 
 
 	/**
-	 *    Calculates the max number of grass instances for this layer.
+	 * @brief Calculates the max number of grass instances for this layer.
 	 * @param page The page to create grass for.
 	 * @param densityFactor The density factor set on the grass loader
 	 * @param volume The volume, in world units, to fill
+	 * @param isAvailable This must be set to true if the grass is currently available.
 	 * @return The max number of grass instances to create.
 	 */
-	virtual unsigned int prepareGrass(const PageInfo& page, float densityFactor, float volume) = 0;
+	virtual unsigned int prepareGrass(const PageInfo& page, float densityFactor, float volume, bool& isAvailable) = 0;
 
 	/** \brief Set the maximum slope a grass of blade can be placed on.
 	\param maxSlopeRatio The maximum slope (h/w ratio) a grass blade is allowed to be placed on.
@@ -596,14 +599,7 @@ public:
 	don't, the grass you see will remain unchanged. */
 	ColorMap *getColorMap() { return colorMap; }
 
-	/**
-	 *    Calculates the max number of grass instances for this layer.
-	 * @param page The page to create grass for.
-	 * @param densityFactor The density factor set on the grass loader
-	 * @param volume The volume, in world units, to fill
-	 * @return The max number of grass instances to create.
-	*/
-	virtual unsigned int prepareGrass(const PageInfo& page, float densityFactor, float volume);
+	virtual unsigned int prepareGrass(const PageInfo& page, float densityFactor, float volume, bool& isAvailable);
 
 
 	/**
@@ -784,6 +780,23 @@ void GrassLoader<TGrassLayer>::frameUpdate()
 	}
 }
 
+template <class TGrassLayer>
+bool GrassLoader<TGrassLayer>::preparePage(PageInfo &page)
+{
+	bool result = true;
+	//Calculate how much grass needs to be added
+	float volume = page.bounds.width() * page.bounds.height();
+	typename std::list<TGrassLayer*>::iterator it;
+	for (it = layerList.begin(); it != layerList.end(); ++it){
+		TGrassLayer *layer = *it;
+		bool isAvailable = false;
+		layer->prepareGrass(page, densityFactor, volume, isAvailable);
+		result = result && isAvailable;
+	}
+	return result;
+}
+
+
 template <class TGrassLayer>
 void GrassLoader<TGrassLayer>::loadPage(PageInfo &page)
 {
@@ -802,9 +815,10 @@ void GrassLoader<TGrassLayer>::loadPage(PageInfo &page)
 
 		//Calculate how much grass needs to be added
 		float volume = page.bounds.width() * page.bounds.height();
-		unsigned int grassCount = layer->prepareGrass(page, densityFactor, volume);
+		bool isAvailable = false;
+		unsigned int grassCount = layer->prepareGrass(page, densityFactor, volume, isAvailable);
 
-		if (grassCount) {
+		if (isAvailable && grassCount) {
 			//The vertex buffer can't be allocated until the exact number of polygons is known,
 			//so the locations of all grasses in this page must be precalculated.
 
@@ -1196,15 +1210,14 @@ Mesh *GrassLoader<TGrassLayer>::generateGrass_CROSSQUADS(PageInfo &page, TGrassL
 
 		float y3, y4;
 		if (heightFunction){
-			if (layer->getMaxSlope() < (Math::Abs(y1 - y2) / (halfScaleX * 2))) {
-				//Degenerate the face
-				x2 = x1;
-				y2 = y1;
-				z2 = z1;
-			}
-
 			y3 = heightFunction(x3, z3, heightFunctionUserData);
 			y4 = heightFunction(x4, z4, heightFunctionUserData);
+			if (layer->getMaxSlope() < (Math::Abs(y3 - y4) / (halfScaleX * 2))) {
+				//Degenerate the face
+				x4 = x3;
+				y4 = y3;
+				z4 = z3;
+			}
 		} else {
 			y3 = 0;
 			y4 = 0;
