@@ -104,6 +104,7 @@ Permission is granted to anyone to use this software for any purpose, including
 
 #include <limits> // numeric_limits<>
 #include <memory>
+#include <set>
 
 #include <OgreRoot.h>
 #include <OgrePrerequisites.h>
@@ -1090,7 +1091,7 @@ private:
 	bool _loaded;		//Flag indicating if page is loaded
 	bool _needsUnload;	//Flag indicating if page needs unloading before next load
 	bool _keepLoaded;	//Flag indicating if the page should not be unloaded
-	std::list<GeometryPage*>::iterator _iter;	//Iterator in loadedList
+//	std::list<GeometryPage*>::iterator _iter;	//Iterator in loadedList
 
 	Ogre::AxisAlignedBox _trueBounds;	//Actual bounding box of the 3D geometry added to this page
 	bool _trueBoundsUndefined;			//Flag indicating if _trueBounds has not been defined yet
@@ -1230,6 +1231,16 @@ loader yourself (if you want to). The PagedGeometry destructor won't do this for
 class PageLoader
 {
 public:
+
+	/**
+	 \brief Called right before loading.
+
+	 This allows the loader to perform asynchronous loading by returning false and then later on
+	 reloading the page.
+	 \return True if the loading should continue, false if it shouldn't.
+	 */
+	virtual bool preparePage(PageInfo &page) { return true; }
+
 	/**
 	\brief This should be overridden to load a specified region of entities.
 	\param page A PageInfo variable which includes boundary information and other useful values.
@@ -1349,7 +1360,7 @@ class GeometryPageManager
 {
 public:
 	/** \brief A std::list of pointers to geometry pages */
-	typedef std::list<GeometryPage*> TPGeometryPages;
+	typedef std::set<GeometryPage*> TPGeometryPages;
 
 	/** \brief Internal function - DO NOT USE */
 	GeometryPageManager(PagedGeometry *mainGeom);
@@ -1542,6 +1553,7 @@ private:
 
 	TPGeometryPages pendingList;	//Pages of geometry to be loaded
 	TPGeometryPages loadedList;	//Pages of geometry already loaded
+	TPGeometryPages reloadList;	//Pages of geometry which should be reloaded
 
 	//Cache settings
 	unsigned long maxCacheInterval;
