@@ -173,8 +173,8 @@ void ImpostorPage::update()
 
 void ImpostorPage::regenerate(Entity *ent)
 {
-	ImpostorTexture *tex = ImpostorTexture::getTexture(NULL, ent);
-	if (tex != NULL)
+	ImpostorTexture *tex = ImpostorTexture::getTexture(nullptr, ent);
+	if (tex)
 		tex->regenerate();
 }
 
@@ -334,7 +334,7 @@ unsigned long ImpostorTexture::GUID = 0;
 //Do not use this constructor yourself - instead, call getTexture()
 //to get/create an ImpostorTexture for an Entity.
 ImpostorTexture::ImpostorTexture(ImpostorPage *group, Entity *entity)
-: loader(0)
+: loader(nullptr)
 {
 	//Store scene manager and entity
 	ImpostorTexture::sceneMgr = group->sceneMgr;
@@ -457,7 +457,7 @@ void ImpostorTexture::renderTextures(bool force)
 #else
 	TexturePtr renderTexture(texture);
 	//if we're not using a file image we need to set up a resource loader, so that the texture is regenerated if it's ever unloaded (such as switching between fullscreen and the desktop in win32)
-	loader = std::auto_ptr<ImpostorTextureResourceLoader>(new ImpostorTextureResourceLoader(*this));
+	loader = std::unique_ptr<ImpostorTextureResourceLoader>(new ImpostorTextureResourceLoader(*this));
 #endif
 	RenderTexture *renderTarget;
 	Camera *renderCamera;
