@@ -498,19 +498,18 @@ void GeometryPageManager::update(unsigned long deltaTime, Vector3 &camPos, Vecto
 					if (distSq >= nearDistSq && distSq < farTransDistSq){
 						//If so, load the geometry immediately
 						_loadPage(blk);
-						loadedList.push_back(blk);
-						blk->_iter = (--loadedList.end());
+						loadedList.insert(blk);
 
 						//And remove it from the pending list if necessary
 						if (blk->_pending){
-							pendingList.remove(blk);
+							pendingList.erase(blk);
 							blk->_pending = false;
 						}
 					} else {
 						//Otherwise, add it to the pending geometry list (if not already)
 						//Pages in then pending list will be loaded later (see below)
 						if (!blk->_pending){
-							pendingList.push_back(blk);
+							pendingList.insert(blk);
 							blk->_pending = true;
 						}
 					}
@@ -567,8 +566,7 @@ void GeometryPageManager::update(unsigned long deltaTime, Vector3 &camPos, Vecto
 			Real distSq = dx * dx + dz * dz;
 			if (distSq <= cacheDistSq){
 				_loadPage(blk);
-				loadedList.push_back(blk);
-				blk->_iter = (--loadedList.end());
+				loadedList.insert(blk);
 
 				enableCache = false;
 				break;
@@ -692,7 +690,9 @@ void GeometryPageManager::reloadGeometry()
 	for (it = loadedList.begin(); it != loadedList.end(); ++it)
 	{
 		GeometryPage *page = *it;
-		_unloadPage(page);
+		_unloadPageDelayed(page);
+		page->_pending = true;
+		pendingList.insert(page);
 	}
 	loadedList.clear();
 }
@@ -708,18 +708,18 @@ void GeometryPageManager::reloadGeometryPage(const Vector3 &point, bool forceLoa
 	if (x >= 0 && z >= 0 && x < geomGridX && z < geomGridZ){
 		GeometryPage *page = _getGridPage(x, z);
 		if (page->_loaded){
-			_unloadPage(page);
-			loadedList.erase(page->_iter);
+			_unloadPageDelayed(page);
+			page->_pending = true;
+			pendingList.insert(page);
 		}
 		if (forceLoadImmediately) {
 			//Load the geometry immediately
 			_loadPage(page);
-			loadedList.push_back(page);
-			page->_iter = (--loadedList.end());
+			loadedList.insert(page);
 
 			//And remove it from the pending list if necessary
 			if (page->_pending){
-				pendingList.remove(page);
+				pendingList.erase(page);
 				page->_pending = false;
 			}
 		}
@@ -754,8 +754,9 @@ void GeometryPageManager::reloadGeometryPages(const Vector3 &center, Real radius
 				Real distSq = distX * distX + distZ * distZ;
 				
 				if (distSq <= radius) {
-					_unloadPage(page);
-					loadedList.erase(page->_iter);
+					_unloadPageDelayed(page);
+					page->_pending = true;
+					pendingList.insert(page);
 				}
 			}
 		}
@@ -781,8 +782,9 @@ void GeometryPageManager::reloadGeometryPages(const TBounds & area)
 		for (int z = z1; z <= z2; ++z) {
 			GeometryPage *page = _getGridPage(x, z);
 			if (page->_loaded){
-				_unloadPage(page);
-				loadedList.erase(page->_iter);
+				_unloadPageDelayed(page);
+				page->_pending = true;
+				pendingList.insert(page);
 			}
 		}
 	}
@@ -817,12 +819,11 @@ void GeometryPageManager::preloadGeometry(const TBounds & area)
 			if (!page->_loaded){
 				//Load the geometry immediately
 				_loadPage(page);
-				loadedList.push_back(page);
-				page->_iter = (--loadedList.end());
+				loadedList.insert(page);
 
 				//And remove it from the pending list if necessary
 				if (page->_pending){
-					pendingList.remove(page);
+					pendingList.erase(page);
 					page->_pending = false;
 				}
 			}
@@ -863,6 +864,13 @@ void GeometryPageManager::_loadPage(GeometryPage *page)
 	info.zIndex = page->_zIndex;
 	info.userData = page->_userData;
 
+	//Check if the page is prepared. If not, should stop now (and rely on the page being loaded again when the background data has been prepared).
+	if (!mainGeom->getPageLoader()->preparePage(info)) {
+		page->_inactiveTime = 0;
+		page->_loaded = true;
+		return;
+	}
+
 	//Check if page needs unloading (if a delayed unload has been issued)
 	if (page->_needsUnload){
 		page->removeEntities();
@@ -940,7 +948,7 @@ void GeometryPageManager::_scrollGridPages(int shiftX, int shiftZ)
 				if (page->_loaded){
 					page->_keepLoaded = false;
 					_unloadPage(page);
-					loadedList.erase(page->_iter);
+					loadedList.erase(page);
 				}
 				page->_centerPoint.x += shiftX * mainGeom->getPageSize();
 				page->_centerPoint.z += shiftZ * mainGeom->getPageSize();
@@ -958,7 +966,7 @@ void GeometryPageManager::_scrollGridPages(int shiftX, int shiftZ)
 					if (page->_loaded){
 						page->_keepLoaded = false;
 						_unloadPageDelayed(page);
-						loadedList.erase(page->_iter);
+						loadedList.erase(page);
 					}
 					scrollBuffer[x] = page;
 				}
@@ -983,7 +991,7 @@ void GeometryPageManager::_scrollGridPages(int shiftX, int shiftZ)
 					if (page->_loaded){
 						page->_keepLoaded = false;
 						_unloadPageDelayed(page);
-						loadedList.erase(page->_iter);
+						loadedList.erase(page);
 					}
 					scrollBuffer[x - initialMidpoint] = page;
 				}
@@ -1007,7 +1015,7 @@ void GeometryPageManager::_scrollGridPages(int shiftX, int shiftZ)
 					if (page->_loaded){
 						page->_keepLoaded = false;
 						_unloadPageDelayed(page);
-						loadedList.erase(page->_iter);
+						loadedList.erase(page);
 					}
 					scrollBuffer[z] = page;
 				}
@@ -1032,7 +1040,7 @@ void GeometryPageManager::_scrollGridPages(int shiftX, int shiftZ)
 					if (page->_loaded){
 						page->_keepLoaded = false;
 						_unloadPageDelayed(page);
-						loadedList.erase(page->_iter);
+						loadedList.erase(page);
 					}
 					scrollBuffer[z - initialMidpoint] = page;
 				}
