@@ -22,15 +22,13 @@
 #include "TerrainManager.h"
 #include "TerrainHandler.h"
 #include "TerrainDefPoint.h"
-#include "ISceneManagerAdapter.h"
+#include "ITerrainAdapter.h"
 #include "components/ogre/EmberOgre.h"
 #include "components/ogre/Convert.h"
 #include "components/ogre/Avatar.h"
 #include "components/ogre/camera/MainCamera.h"
 #include "components/ogre/GUIManager.h"
 #include "components/ogre/EmberEntity.h"
-#include "components/ogre/WorldEmberEntity.h"
-#include "components/ogre/EmberEntityFactory.h"
 #include "components/ogre/MousePicker.h"
 #include "components/ogre/World.h"
 
@@ -50,7 +48,8 @@
 #include <OgreEntity.h>
 
 #include <sigc++/bind.h>
-#include <boost/smart_ptr.hpp>
+
+#include <memory>
 
 namespace Ember
 {
@@ -61,12 +60,10 @@ namespace Terrain
 {
 
 BasePointUserObject::BasePointUserObject(const Domain::TerrainPosition terrainPosition, const Mercator::BasePoint& basePoint, Ogre::SceneNode* basePointMarkerNode) :
-	mBasePoint(basePoint), mBasePointMarkerNode(basePointMarkerNode), mPosition(terrainPosition)
+		mBasePoint(basePoint), mBasePointMarkerNode(basePointMarkerNode), mPosition(terrainPosition), mCanonicalHeight(mBasePoint.height()), mIsMoving(false)
 {
 }
 
-
-
 const Mercator::BasePoint& BasePointUserObject::getBasePoint() const
 {
 	return mBasePoint;
@@ -96,7 +93,7 @@ float BasePointUserObject::getHeight() const
 void BasePointUserObject::translate(Ogre::Real verticalMovement)
 {
 	getBasePointMarkerNode()->translate(Ogre::Vector3(0, verticalMovement, 0));
-	markAsMoved();
+	updateMarking();
 	EventUpdatedPosition();
 }
 
@@ -104,24 +101,42 @@ void BasePointUserObject::setHeight(Ogre::Real height)
 {
 	const Ogre::Vector3& position = getBasePointMarkerNode()->getPosition();
 	getBasePointMarkerNode()->setPosition(position.x, height, position.z);
-	markAsMoved();
+	updateMarking();
 	EventUpdatedPosition();
 }
 
-void BasePointUserObject::markAsMoved()
+void BasePointUserObject::updateMarking()
 {
-	Ogre::Entity* entity = static_cast<Ogre::Entity*> (getBasePointMarkerNode()->getAttachedObject(0));
-	entity->setMaterialName("/global/authoring/point/moved");
+	Ogre::Entity* entity = static_cast<Ogre::Entity*>(getBasePointMarkerNode()->getAttachedObject(0));
+	try {
+		if (mIsMoving) {
+			entity->setMaterialName("/global/authoring/point/moving");
+		} else {
+			if (mCanonicalHeight != getBasePointMarkerNode()->_getDerivedPosition().y) {
+				entity->setMaterialName("/global/authoring/point/moved");
+			} else {
+				entity->setMaterialName("/global/authoring/point");
+			}
+		}
+	} catch (const std::exception& ex) {
+		S_LOG_WARNING("Could not set new marker material. This is not fatal."<<ex);
+	}
 }
 
 void BasePointUserObject::resetMarking()
 {
-	Ogre::Entity* entity = static_cast<Ogre::Entity*> (getBasePointMarkerNode()->getAttachedObject(0));
-	entity->setMaterialName("/global/authoring/point");
+	mCanonicalHeight = mBasePoint.height();
+	updateMarking();
+}
+
+void BasePointUserObject::markAsMoving(bool isMoving)
+{
+	mIsMoving = isMoving;
+	updateMarking();
 }
 
 BasePointPickListener::BasePointPickListener(TerrainEditorOverlay& overlay) :
-	mOverlay(overlay), mPickedUserObject(0)
+		mOverlay(overlay), mPickedUserObject(0)
 {
 
 }
@@ -130,16 +145,29 @@ void BasePointPickListener::processPickResult(bool& continuePicking, Ogre::RaySc
 {
 	if (entry.movable) {
 		Ogre::MovableObject* pickedMovable = entry.movable;
-		if (pickedMovable->isVisible() && pickedMovable->getUserAny().getType() == typeid(BasePointUserObject::SharedPtr)) {
-			mPickedUserObject = Ogre::any_cast<BasePointUserObject::SharedPtr> (pickedMovable->getUserAny()).get();
+		if (pickedMovable->isVisible() && pickedMovable->getUserObjectBindings().getUserAny().getType() == typeid(BasePointUserObject::SharedPtr)) {
 			continuePicking = false;
+			if (mousePickerArgs.pickType == MPT_PRESS) {
+				mPickedUserObject = Ogre::any_cast<BasePointUserObject::SharedPtr>(pickedMovable->getUserObjectBindings().getUserAny()).get();
+			}
 		}
 	}
 }
 
-void BasePointPickListener::initializePickingContext()
+void BasePointPickListener::processDelayedPick(const MousePickerArgs& mousePickerArgs)
 {
-	mPickedUserObject = 0;
+	//Don't process any delayed picks.
+}
+
+
+void BasePointPickListener::initializePickingContext(bool& willParticipate, unsigned int& queryMask, const MousePickerArgs& pickArgs)
+{
+	//We will only react on press events, but we want do silence click and pressed events if they happen with our markers too.
+	if (pickArgs.pickType == MPT_PRESS || pickArgs.pickType == MPT_CLICK || pickArgs.pickType == MPT_PRESSED) {
+		willParticipate = true;
+		queryMask = MousePicker::CM_UNDEFINED;
+		mPickedUserObject = 0;
+	}
 }
 
 void BasePointPickListener::endPickingContext(const MousePickerArgs& mousePickerArgs)
@@ -150,7 +178,7 @@ void BasePointPickListener::endPickingContext(const MousePickerArgs& mousePicker
 }
 
 TerrainEditBasePointMovement::TerrainEditBasePointMovement(Ogre::Real verticalMovement, Domain::TerrainPosition position) :
-	mVerticalMovement(verticalMovement), mPosition(position)
+		mVerticalMovement(verticalMovement), mPosition(position)
 {
 }
 
@@ -164,8 +192,8 @@ const Domain::TerrainPosition& TerrainEditBasePointMovement::getPosition() const
 	return mPosition;
 
 }
-TerrainEditorOverlay::TerrainEditorOverlay(TerrainEditor& editor, Ogre::SceneManager& sceneManager, Ogre::SceneNode& worldSceneNode, TerrainManager& manager, Camera::MainCamera& camera, std::map<int, std::map<int, Mercator::BasePoint> >& basePoints) :
-	mEditor(editor), mSceneManager(sceneManager), mWorldSceneNode(worldSceneNode), mManager(manager), mCamera(camera), mOverlayNode(0), mPickListener(*this), mCurrentUserObject(0)
+TerrainEditorOverlay::TerrainEditorOverlay(TerrainEditor& editor, Ogre::SceneManager& sceneManager, Ogre::SceneNode& worldSceneNode, TerrainManager& manager, Camera::MainCamera& camera, std::map<int, std::map<int, Mercator::BasePoint>>& basePoints) :
+		mEditor(editor), mSceneManager(sceneManager), mWorldSceneNode(worldSceneNode), mManager(manager), mCamera(camera), mOverlayNode(0), mPickListener(*this), mCurrentUserObject(0)
 {
 	createOverlay(basePoints, worldSceneNode);
 }
@@ -179,7 +207,7 @@ TerrainEditorOverlay::~TerrainEditorOverlay()
 	}
 	//TODO: also delete user objects
 	if (mOverlayNode) {
-		Ogre::SceneNode* parent = static_cast<Ogre::SceneNode*> (mOverlayNode->getParent());
+		Ogre::SceneNode* parent = static_cast<Ogre::SceneNode*>(mOverlayNode->getParent());
 		if (parent) {
 			parent->removeAndDestroyChild(mOverlayNode->getName());
 		} else {
@@ -190,7 +218,7 @@ TerrainEditorOverlay::~TerrainEditorOverlay()
 	mCamera.removeWorldPickListener(&mPickListener);
 }
 
-void TerrainEditorOverlay::createOverlay(std::map<int, std::map<int, Mercator::BasePoint> >& basePoints, Ogre::SceneNode& worldSceneNode)
+void TerrainEditorOverlay::createOverlay(std::map<int, std::map<int, Mercator::BasePoint>>& basePoints, Ogre::SceneNode& worldSceneNode)
 {
 
 	mOverlayNode = worldSceneNode.createChildSceneNode();
@@ -207,6 +235,9 @@ void TerrainEditorOverlay::createOverlay(std::map<int, std::map<int, Mercator::B
 				entity = mSceneManager.createEntity(ss.str(), "3d_objects/primitives/models/sphere.mesh");
 				//start out with a normal material
 				entity->setMaterialName("/global/authoring/point");
+				//The material is made to ignore depth checks, so if we put it in a later queue we're
+				//making sure that the marker is drawn on top of everything else, making it easier to interact with.
+				entity->setRenderQueueGroup(Ogre::RENDER_QUEUE_9);
 				entity->setRenderingDistance(300);
 				entity->setQueryFlags(MousePicker::CM_UNDEFINED);
 			} catch (const std::exception& ex) {
@@ -230,7 +261,7 @@ void TerrainEditorOverlay::createOverlay(std::map<int, std::map<int, Mercator::B
 			basepointNode->attachObject(entity);
 
 			BasePointUserObject* userObject = new BasePointUserObject(Domain::TerrainPosition(x, y), basepoint, basepointNode);
-			entity->setUserAny(Ogre::Any(BasePointUserObject::SharedPtr(userObject)));
+			entity->getUserObjectBindings().setUserAny(Ogre::Any(BasePointUserObject::SharedPtr(userObject)));
 
 			//store the base point user object
 			std::stringstream ss_;
@@ -266,6 +297,7 @@ void TerrainEditorOverlay::pickedBasePoint(BasePointUserObject* userObject)
 	assert(userObject);
 	mCurrentUserObject = userObject;
 	catchInput();
+	mCurrentUserObject->markAsMoving(true);
 	EventPickedBasePoint.emit(userObject);
 }
 
@@ -276,6 +308,7 @@ bool TerrainEditorOverlay::injectMouseMove(const MouseMotion& motion, bool& free
 	if (Input::getSingleton().isKeyDown(SDLK_RSHIFT) || Input::getSingleton().isKeyDown(SDLK_LSHIFT)) {
 		multiplier *= 5;
 	}
+
 	assert(mCurrentUserObject);
 	float translation(motion.yRelativeMovement * multiplier);
 	mCurrentUserObject->translate(translation);
@@ -315,7 +348,7 @@ bool TerrainEditorOverlay::injectMouseButtonDown(const Input::MouseButton& butto
 	return true;
 }
 
-bool TerrainEditorOverlay::injectChar(char character)
+bool TerrainEditorOverlay::injectChar(int character)
 {
 	return true;
 }
@@ -347,6 +380,7 @@ void TerrainEditorOverlay::releaseInput()
 void TerrainEditorOverlay::createAction(bool alsoCommit)
 {
 	if (mCurrentUserObject) {
+		mCurrentUserObject->markAsMoving(false);
 		//lets get how much it moved
 		float distance = mCurrentUserObject->getBasePointMarkerNode()->getPosition().y - mCurrentUserObject->getBasePoint().height();
 		//only register an action if it has been moved
@@ -384,7 +418,7 @@ void TerrainEditorOverlay::sendChangesToServer()
 	mManager.getBasePoints(slot);
 }
 
-void TerrainEditorOverlay::sendChangesToServerWithBasePoints(std::map<int, std::map<int, Mercator::BasePoint> >& basePoints)
+void TerrainEditorOverlay::sendChangesToServerWithBasePoints(std::map<int, std::map<int, Mercator::BasePoint>>& basePoints)
 {
 
 	try {
@@ -411,7 +445,7 @@ void TerrainEditorOverlay::sendChangesToServerWithBasePoints(std::map<int, std::
 			Mercator::BasePoint bp;
 			WFMath::CoordType basepointX = I->second.x();
 			WFMath::CoordType basepointY = I->second.y();
-			getBasePoint(basePoints, static_cast<int> (basepointX), static_cast<int> (basepointY), bp);
+			getBasePoint(basePoints, static_cast<int>(basepointX), static_cast<int>(basepointY), bp);
 
 			Atlas::Message::ListType & point = (pointMap[I->first] = Atlas::Message::ListType(3)).asList();
 			point[0] = (Atlas::Message::FloatType)(I->second.x());
@@ -462,8 +496,6 @@ bool TerrainEditorOverlay::getVisible() const
 	return mOverlayNode != 0 && mOverlayNode->isInSceneGraph();
 }
 
-
-
 bool TerrainEditorOverlay::undoLastAction()
 {
 	if (mActions.size() > 0) {
@@ -497,15 +529,16 @@ void TerrainEditorOverlay::commitAction(const TerrainEditAction& action, bool re
 	mManager.getBasePoints(slot);
 }
 
-void TerrainEditorOverlay::commitActionWithBasePoints(BasePointStore& basePoints, const TerrainEditAction action, bool reverse) {
+void TerrainEditorOverlay::commitActionWithBasePoints(BasePointStore& basePoints, const TerrainEditAction action, bool reverse)
+{
 
 	TerrainDefPointStore pointStore;
 
 	std::set<TerrainPage*> pagesToUpdate;
 	for (TerrainEditAction::MovementStore::const_iterator I = action.getMovements().begin(); I != action.getMovements().end(); ++I) {
 		Mercator::BasePoint bp;
-		int basepointX = static_cast<int> (I->getPosition().x());
-		int basepointY = static_cast<int> (I->getPosition().y());
+		int basepointX = static_cast<int>(I->getPosition().x());
+		int basepointY = static_cast<int>(I->getPosition().y());
 		getBasePoint(basePoints, basepointX, basepointY, bp);
 		//check if we should do a reverse action (which is done when an action is undone)
 		bp.height() = bp.height() + (reverse ? -I->getVerticalMovement() : I->getVerticalMovement());
@@ -556,7 +589,6 @@ void TerrainEditorOverlay::commitActionWithBasePoints(BasePointStore& basePoints
 	// 		(*I)->updateAllShaderTextures();
 	// 	}
 
-
 	// 	std::set<Ogre::PagingLandScapeData2D*> dataStore;
 	// 	//reload all affected tiles
 	// 	for (std::set<Ogre::PagingLandScapeTile*>::iterator I = tilesToUpdate.begin(); I != tilesToUpdate.end(); ++I) {
@@ -586,13 +618,11 @@ void TerrainEditorOverlay::commitActionWithBasePoints(BasePointStore& basePoints
 	//sceneMgr->getPageManager()->load();
 	// 	TerrainManager->getAdapter()->reloadAllPages();
 
-
 	// 	updateEntityPositions(pagesToUpdate);
 
-
 }
 
-bool TerrainEditorOverlay::getBasePoint(const std::map<int, std::map<int, Mercator::BasePoint> >& basePoints, int x, int y, Mercator::BasePoint& z) const
+bool TerrainEditorOverlay::getBasePoint(const std::map<int, std::map<int, Mercator::BasePoint>>& basePoints, int x, int y, Mercator::BasePoint& z) const
 {
 	Mercator::Terrain::Pointstore::const_iterator I = basePoints.find(x);
 	if (I == basePoints.end()) {
