@@ -20,6 +20,9 @@
 #define TERRAINHANDLER_H_
 
 #include "Types.h"
+#include "domain/IHeightProvider.h"
+
+#include <wfmath/vector.h>
 
 #include <sigc++/trackable.h>
 #include <sigc++/signal.h>
@@ -38,6 +41,7 @@ namespace Mercator {
 
 namespace Ember
 {
+class TimeFrame;
 namespace Tasks {
 	class TaskQueue;
 }
@@ -55,7 +59,7 @@ class TerrainArea;
 class TerrainMod;
 class TerrainLayerDefinition;
 class TerrainPageSurfaceLayer;
-class ISceneManagerAdapter;
+class ITerrainAdapter;
 class ITerrainPageBridge;
 class ICompilerTechniqueProvider;
 class HeightMap;
@@ -78,7 +82,7 @@ class PlantPopulator;
  *
  * Most terrain handling functions are implemented as Tasks, which are performed on a Task queue. As much work as possible is done in background threads, and then synced with both Ogre and the Mercator representations in the main thread.
  */
-class TerrainHandler : public virtual sigc::trackable
+class TerrainHandler : public virtual sigc::trackable, public Ember::Domain::IHeightProvider
 {
 public:
 
@@ -87,13 +91,26 @@ public:
 	 * @param pageIndexSize The size of one side of a page, in indices.
 	 * @param compilerTechniqueProvider Provider for terrain surface compilation techniques.
 	 */
-	TerrainHandler(int pageIndexSize, ICompilerTechniqueProvider& compilerTechniqueProvider);
+	TerrainHandler(unsigned int pageIndexSize, ICompilerTechniqueProvider& compilerTechniqueProvider);
 
 	/**
 	 * @brief Dtor.
 	 */
 	virtual ~TerrainHandler();
 
+	/**
+	 * @brief Shuts down the handler; call this before deleting the instance.
+	 *
+	 * This will mainly deactivate the task queue. This might be important if there are outstanding threads waiting for
+	 * it to complete.
+	 */
+	void shutdown();
+
+	/**
+	 * @brief Sets the size of the width of one page, in indices. This must be a power of two + 1 and at least 65.
+	 * @param pageSize The number of vertices along one side of a page. Must be a power of two + 1 and at least 65.
+	 */
+	void setPageSize(unsigned int pageSize);
 
 	/**
 	 * @brief Returns the height at the specified position in the world.
@@ -219,12 +236,20 @@ public:
 	int getPageIndexSize() const;
 
 	/**
-	 * @brief Adds a new page to the manager,
+	 * @brief Adds a new page to the handler.
 	 *
 	 * @param page A terrain page instance.
 	 */
 	void addPage(TerrainPage* page);
 
+
+	/**
+	 * @brief Destroys a page, removing it from the handler and deleting the instance.
+	 *
+	 * @param page A terrain page instance.
+	 */
+	void destroyPage(TerrainPage* page);
+
 	/**
 	 * @brief Reloads the terrain found at the specified positions.
 	 *
@@ -239,7 +264,7 @@ public:
 	 * Calling this method will update both the internal Mercator heightfield data as well as the Ogre graphical representation, and will trigger updates of any dependent featues (such as the foliage).
 	 * @param positions A vector of terrain areas, in world space. The terrain found at these areas will be reloaded.
 	 */
-	void reloadTerrain(const std::vector<WFMath::AxisBox<2> >& areas);
+	void reloadTerrain(const std::vector<WFMath::AxisBox<2>>& areas);
 
 	/**
 	 * @brief Gets all currently defined basepoints asynchronously.
@@ -247,7 +272,7 @@ public:
 	 * The call to the callback will happen in the main thread.
 	 * @param asyncCallback The callback which will be called when all base points have been fetched.
 	 */
-	void getBasePoints(sigc::slot<void, std::map<int, std::map<int, Mercator::BasePoint> >& >& asyncCallback);
+	void getBasePoints(sigc::slot<void, std::map<int, std::map<int, Mercator::BasePoint>>& >& asyncCallback);
 
 	/**
 	 * @brief Sets the lightning instance to use.
@@ -300,8 +325,10 @@ public:
 	 * @brief Polls the tasks queue.
 	 *
 	 * Call this at a regular interval.
+	 *
+	 * @param timeFrame The time frame allowed for polling.
 	 */
-	void pollTasks();
+	void pollTasks(const TimeFrame& timeFrame);
 
 	/**
 	 * @brief Updates all pages.
@@ -329,7 +356,7 @@ public:
 	 * The first parameter is the areas which are affected by the change.
 	 * The second parameter is the pages that will be updated.
 	 */
-	sigc::signal<void, const std::vector<WFMath::AxisBox<2> >&, const std::set<TerrainPage*>&> EventBeforeTerrainUpdate;
+	sigc::signal<void, const std::vector<WFMath::AxisBox<2>>&, const std::set<TerrainPage*>&> EventBeforeTerrainUpdate;
 
 	/**
 	 * @brief Emitted after the terrain geometry has changed.
@@ -338,21 +365,26 @@ public:
 	 * The first parameter is the areas which are affected by the change.
 	 * The second parameter is the pages that were updated.
 	 */
-	sigc::signal<void, const std::vector<WFMath::AxisBox<2> >&, const std::set<TerrainPage*>&> EventAfterTerrainUpdate;
+	sigc::signal<void, const std::vector<WFMath::AxisBox<2>>&, const std::set<TerrainPage*>&> EventAfterTerrainUpdate;
 
 	/**
 	 * @brief Emitted when the size of the world has changed.
 	 */
 	sigc::signal<void> EventWorldSizeChanged;
 
+	/**
+	 * @brief Emitted after a terrain material has been recompiled.
+	 */
+	sigc::signal<void, TerrainPage* > EventTerrainMaterialRecompiled;
+
 protected:
 
-	typedef std::map<Domain::TerrainIndex, boost::shared_ptr<ITerrainPageBridge> > PageBridgeStore;
+	typedef std::map<Domain::TerrainIndex, std::shared_ptr<ITerrainPageBridge>> PageBridgeStore;
 
 	/**
 	 * @brief The size in indices of one side of a page.
 	 */
-	int mPageIndexSize;
+	unsigned int mPageIndexSize;
 
 	/**
 	 * @brief The terrain page surface compiler technique provider which allows Ogre Material instances to be generated for the terrain pages.
@@ -362,7 +394,7 @@ protected:
 	/**
 	@brief Information about the world, such as size and number of pages.
 	*/
-	std::auto_ptr<TerrainInfo> mTerrainInfo;
+	std::unique_ptr<TerrainInfo> mTerrainInfo;
 
 	/**
 	 * @brief The main Mercator terrain instance, which holds all of the Mercator terrain structures.
@@ -466,6 +498,14 @@ protected:
 	 */
 	SegmentManager* mSegmentManager;
 
+	/**
+	 * @brief The angle used when lighting for precomputed shadows was last updated.
+	 *
+	 * We need to keep track of this in order to know when to update the precomputed shadows (once every hour).
+	 * This is only of used when using the fixed function pipeline (which requries precomputed shadows).
+	 */
+	WFMath::Vector<3> mLastLightingUpdateAngle;
+
 	/**
 	 * @brief Marks a shader for update, to be updated on the next batch, normally a frameEnded event.
 	 *
@@ -513,6 +553,14 @@ protected:
 	 * Note that this only regenerates the Mercator height map, and won't update the Ogre representation.
 	 */
 	void buildHeightmap();
+
+	/**
+	 * @brief Called each frame.
+	 * @param
+	 * @param
+	 */
+	void frameProcessed(const TimeFrame&, unsigned int);
+
 };
 
 inline const std::list<TerrainShader*>& TerrainHandler::getBaseShaders() const
