@@ -11,12 +11,12 @@ end
 
 function SettingsRestartDialog:buildWidget()
 	self.widget = guiManager:createWidget()
-	self.widget:loadMainSheet("SettingsRestartDialog.layout", "SettingsRestartDialog/")
+	self.widget:loadMainSheet("SettingsRestartDialog.layout", "SettingsRestartDialog")
 	
-	self.window = self.widget:getWindow("MainWindow")
+	self.window = self.widget:getMainWindow()
 	self.window:subscribeEvent("CloseClicked", self.CloseClicked, self)
 	
-	self.okButton = self.widget:getWindow("MainWindow/OKButton")
+	self.okButton = self.widget:getWindow("OKButton")
 	self.okButton:subscribeEvent("Clicked", self.CloseClicked, self)
 	
 	-- make it a modal window to prevent user from missing the info
@@ -49,14 +49,14 @@ end
 
 function SettingsUnappliedChangesDialog:buildWidget()
 	self.widget = guiManager:createWidget()
-	self.widget:loadMainSheet("SettingsUnappliedChangesDialog.layout", "SettingsUnappliedChangesDialog/")
+	self.widget:loadMainSheet("SettingsUnappliedChangesDialog.layout", "SettingsUnappliedChangesDialog")
 	
-	self.window = self.widget:getWindow("MainWindow")
+	self.window = self.widget:getMainWindow()
 	
-	self.applyButton = self.widget:getWindow("MainWindow/ApplyButton")
+	self.applyButton = self.widget:getWindow("ApplyButton")
 	self.applyButton:subscribeEvent("Clicked", self.ApplyClicked, self)
 	
-	self.discardButton = self.widget:getWindow("MainWindow/DiscardButton")
+	self.discardButton = self.widget:getWindow("DiscardButton")
 	self.discardButton:subscribeEvent("Clicked", self.DiscardClicked, self)
 	
 	-- make it a modal window to prevent user from missing the info
@@ -93,21 +93,21 @@ SettingsWidget = {}
 function SettingsWidget:buildWidget()
 	
 	self.widget = guiManager:createWidget()
-	self.widget:loadMainSheet("Settings.layout", "Settings/")
+	self.widget:loadMainSheet("Settings.layout", "Settings")
 	
-	self.window = self.widget:getWindow("MainWindow")
+	self.window = self.widget:getMainWindow()
 	self.window:subscribeEvent("CloseClicked", self.CloseClicked, self)
 	
-	self.okButton = self.widget:getWindow("MainWindow/OKButton")
+	self.okButton = self.widget:getWindow("OKButton")
 	self.okButton:subscribeEvent("Clicked", self.OkClicked, self)
 	
-	self.applyButton = self.widget:getWindow("MainWindow/ApplyButton")
+	self.applyButton = self.widget:getWindow("ApplyButton")
 	self.applyButton:subscribeEvent("Clicked", self.ApplyClicked, self)
 	
-	self.cancelButton = self.widget:getWindow("MainWindow/CancelButton")
+	self.cancelButton = self.widget:getWindow("CancelButton")
 	self.cancelButton:subscribeEvent("Clicked", self.CloseClicked, self)
 	
-	self.tabs = self.widget:getWindow("MainWindow/Tabs")
+	self.tabs = self.widget:getWindow("Tabs")
 	
 	self:buildSettingsUi()
 	
@@ -215,7 +215,7 @@ function SettingsWidget:buildSettingsUi()
 					section = "general",
 					key = "desiredfps",
 					
-					representationFactory = function(value) return Representations.VarconfSliderRepresentation:new_local(value, 100) end,
+					representationFactory = function(value) return Representations.VarconfSliderRepresentation:new_local(value, 150) end,
 				},
 				{
 					label = "Suppress Ogre settings window",
@@ -234,6 +234,15 @@ function SettingsWidget:buildSettingsUi()
 			
 			contents =
 			{
+				{
+					label = "Automatic Graphics Settings",
+					helpString = "Sets whether the graphics settings are managed automatically",
+					
+					section = "graphics",
+					key = "autoadjust",
+					
+					representationFactory = function(value) return Representations.VarconfCheckboxRepresentation:new_local(value) end,
+				},
 				{
 					label = "Level",
 					helpString = "General graphics level",
@@ -244,6 +253,24 @@ function SettingsWidget:buildSettingsUi()
 					representationFactory = function(value) return Representations.VarconfStringComboboxRepresentation:new_local(value, true) end,
 					suggestions = {"low", "medium", "high"}
 				},
+				{
+					label = "Render Distance",
+					helpString = "Render distance in percentage of maximum distance",
+					
+					section = "graphics",
+					key = "renderdistance",
+					
+					representationFactory = function(value) return Representations.VarconfSliderRepresentation:new_local(value, 100) end,
+				},
+				{
+					label = "Level of detail bias",
+					helpString = "Affects the detail level of all entities displayed",
+					
+					section = "graphics",
+					key = "lodbias",
+					
+					representationFactory = function(value) return Representations.VarconfSliderRepresentation:new_local(value, 100) end,
+				},
 				--FIXME: Removed from Ember, not effective
 				--[[{
 					label = "Fresnel water",
@@ -263,6 +290,25 @@ function SettingsWidget:buildSettingsUi()
 					
 					representationFactory = function(value) return Representations.VarconfCheckboxRepresentation:new_local(value) end,
 				},
+				{
+					label = "Foliage Density",
+					helpString = "Density of foliage in percentage",
+					
+					section = "graphics",
+					key = "foliagedensity",
+					
+					representationFactory = function(value) return Representations.VarconfSliderRepresentation:new_local(value, 100) end,
+				},
+				{
+					label = "Foliage Distance",
+					helpString = "Foliage Distance in a percentage of max value",
+					
+					section = "graphics",
+					key = "foliagefardistance",
+					
+					representationFactory = function(value) return Representations.VarconfSliderRepresentation:new_local(value, 100) end,
+				},
+				
 				--FIXME: not effective for now
 				--[[{
 					label = "Double buffered",
@@ -292,6 +338,15 @@ function SettingsWidget:buildSettingsUi()
 					
 					representationFactory = function(value) return Representations.VarconfSliderRepresentation:new_local(value, 1000) end,
 				},
+				{
+					label = "Shadow level of detail",
+					helpString = "Affects the model used to generate the shadow, lower values result in overall lower detail shadows",
+					
+					section = "shadows",
+					key = "shadowlodbias",
+					
+					representationFactory = function(value) return Representations.VarconfSliderRepresentation:new_local(value, 100) end,
+				},
 				{
 					label = "Use aggressive focus region",
 					helpString = "Sets whether or not to use the more aggressive approach to deciding on the focus region or not.",
@@ -310,7 +365,28 @@ function SettingsWidget:buildSettingsUi()
 					
 					representationFactory = function(value) return Representations.VarconfStringComboboxRepresentation:new_local(value, true) end,
 					suggestions = {"ShaderNormalMapped", "Shader", "Base"},
-				}
+				},
+				{
+					label = "Terrain page size",
+					helpString = "The size of single terrain pages. Affects how fast pages are loaded at the start of the game and how much memory is used.",
+
+					section = "terrain",
+					key = "pagesize",
+
+					representationFactory = function(value) return Representations.VarconfIntComboboxRepresentation:new_local(value, true) end,
+					suggestions = {64, 128, 256, 512},
+				},
+				{
+					label = "Terrain load radius",
+					helpString = "The distance from the camera at which terrain pages are loaded. Affects how fast the initial loading is as well as the memory usage and performance in-game.",
+
+					section = "terrain",
+					key = "loadradius",
+
+					representationFactory = function(value) return Representations.VarconfIntComboboxRepresentation:new_local(value, true) end,
+					suggestions = {100, 300, 500},
+				},
+
 			},
 		},
 		{
@@ -380,6 +456,15 @@ function SettingsWidget:buildSettingsUi()
 					section = "input",
 					key = "adjusttoterrain",
 					
+					representationFactory = function(value) return Representations.VarconfCheckboxRepresentation:new_local(value) end,
+				},
+				{
+					label = "Auto movement mode",
+					helpString = "If true, Ember will automatically switch to movement mode after entering the world",
+					
+					section = "input",
+					key = "automovementmode",
+					
 					representationFactory = function(value) return Representations.VarconfCheckboxRepresentation:new_local(value) end,
 				},
 			},
@@ -502,7 +587,7 @@ function SettingsWidget:buildSettingsUi()
 		end
 		
 		local wnd = self:buildUiFor(category)
-		self.tabs:addChildWindow(wnd)
+		self.tabs:addChild(wnd)
 	end
 end
 
@@ -521,8 +606,8 @@ end
 function SettingsWidget:buildUiFor(category)
 	local ret = CEGUI.WindowManager:getSingleton():createWindow("EmberLook/ScrollablePane")
 	ret:setText(category.label)
-	ret:setProperty("UnifiedPosition", "{{0.0, 0.0}, {0.0, 0.0}}")
-	ret:setProperty("UnifiedSize", "{{1.0, 0.0}, {1.0, 0.0}}")
+	ret:setProperty("Position", "{{0.0, 0.0}, {0.0, 0.0}}")
+	ret:setProperty("Size", "{{1.0, 0.0}, {1.0, 0.0}}")
 	
 	-- you can think of the vertical layout container as a VBoxLayout in GTK or Vertical Layout in Qt4
 	-- it simply puts its elements as tightly packed as possible after each other vertically
@@ -530,9 +615,9 @@ function SettingsWidget:buildUiFor(category)
 	local vbox = CEGUI.WindowManager:getSingleton():createWindow("VerticalLayoutContainer")
 	local description = CEGUI.WindowManager:getSingleton():createWindow("EmberLook/StaticText")
 	description:setText(category.description)
-	description:setProperty("UnifiedSize", "{{1.0, -1.0}, {0.0, 50.0}}")
+	description:setProperty("Size", "{{1.0, -1.0}, {0.0, 50.0}}")
 	description:setProperty("HorzFormatting", "WordWrapLeftAligned")
-	vbox:addChildWindow(description)
+	vbox:addChild(description)
 	
 	local configService = emberServices:getConfigService()
 	
@@ -552,9 +637,9 @@ function SettingsWidget:buildUiFor(category)
 		
 		local label = CEGUI.WindowManager:getSingleton():createWindow("EmberLook/StaticText")
 		label:setText(data.label)
-		label:setProperty("UnifiedSize", "{{0.4, 0.0}, {0.0, 30.0}}")
+		label:setProperty("Size", "{{0.4, 0.0}, {0.0, 30.0}}")
 		label:setProperty("FrameEnabled", "False")
-		hbox:addChildWindow(label)
+		hbox:addChild(label)
 		data.labelWnd = label
 		
 		for _, suggestion in ipairs(suggestions) do
@@ -562,8 +647,8 @@ function SettingsWidget:buildUiFor(category)
 		end
 		
 		local representationGuiRoot = representation:getGuiRoot()
-		representationGuiRoot:setProperty("UnifiedSize", "{{0.6, 0}, {0.0, 30.0}}")
-		hbox:addChildWindow(representationGuiRoot)
+		representationGuiRoot:setProperty("Size", "{{0.6, 0}, {0.0, 30.0}}")
+		hbox:addChild(representationGuiRoot)
 		
 		hbox:setTooltipText(data.helpString)
 		
@@ -571,12 +656,12 @@ function SettingsWidget:buildUiFor(category)
 		-- In the end I decided to show help strings as a tooltip to save space
 		--local helpStringLabel = CEGUI.WindowManager:getSingleton():createWindow("EmberLook/StaticText")
 		--helpStringLabel:setText(data.helpString)
-		--helpStringLabel:setProperty("UnifiedSize", "{{0.5, -1.0}, {0.0, 30.0}}")
+		--helpStringLabel:setProperty("Size", "{{0.5, -1.0}, {0.0, 30.0}}")
 		--helpStringLabel:setProperty("FrameEnabled", "False")
 		--helpStringLabel:setProperty("HorzFormatting", "WordWrapLeftAligned")
-		--hbox:addChildWindow(helpStringLabel)
+		--hbox:addChild(helpStringLabel)
 		
-		vbox:addChildWindow(hbox)
+		vbox:addChild(hbox)
 		
 		-- store the representation in data so that we can query it later (to get current value)
 		data.representation = representation
@@ -590,7 +675,7 @@ function SettingsWidget:buildUiFor(category)
 		connect(self.connectors, representation:getEventValueChangedSignal(), valueChangedCall)
 	end
 	
-	ret:addChildWindow(vbox)
+	ret:addChild(vbox)
 	return ret
 end
 
@@ -746,9 +831,7 @@ function SettingsWidget:EventChangedConfigItem(section, key)
 	self:RepresentationValueChanged(section, key)
 end
 
-
-connect(connectors, emberOgre.EventGUIManagerInitialized, function(guiManager)
-
+local settingsInit = function()
 	local settingsWidget = {connectors={}}
 	setmetatable(settingsWidget, {__index = SettingsWidget})
 	settingsWidget:buildWidget()
@@ -757,4 +840,5 @@ connect(connectors, emberOgre.EventGUIManagerInitialized, function(guiManager)
 		settingsWidget:shutdown()
 		settingsWidget = nil
 	end)
-end)
+end
+settingsInit()
