@@ -8,7 +8,7 @@ IndirectRenderingChecker = {}
 
 if Ember.OgreView.OgreInfo:isIndirect() then
 	IndirectRenderingChecker.widget = guiManager:createWidget()
-	IndirectRenderingChecker.widget:loadMainSheet("IndirectRenderingChecker.layout", "IndirectRenderingChecker/")
+	IndirectRenderingChecker.widget:loadMainSheet("IndirectRenderingChecker.layout", "IndirectRenderingChecker")
 	IndirectRenderingChecker.widget:show()
 	IndirectRenderingChecker.widget:getMainWindow():setAlwaysOnTop(true)
 	
