@@ -22,21 +22,32 @@
 
 #include "TimeFrame.h"
 
-#include "framework/Time.h"
-
 namespace Ember
 {
-TimeFrame::TimeFrame(long long timeSliceMilliseconds) :
-	mThresholdMilliseconds(Time::currentTimeMillis() + timeSliceMilliseconds)
+TimeFrame::TimeFrame(const boost::posix_time::time_duration timeSlice) :
+		mStartTime(boost::posix_time::microsec_clock::local_time()), mTimeSlice(timeSlice)
 {
 }
 
 bool TimeFrame::isTimeLeft() const
 {
-	if (Time::currentTimeMillis() > mThresholdMilliseconds) {
+	if ((boost::posix_time::microsec_clock::local_time() - mStartTime) > mTimeSlice) {
 		return false;
 	}
 	return true;
 }
 
+boost::posix_time::time_duration TimeFrame::getRemainingTime() const
+{
+	static const boost::posix_time::time_duration zeroTime = boost::posix_time::microseconds(0);
+	const boost::posix_time::time_duration remaining = getElapsedTime() - mTimeSlice;
+
+	return std::max<boost::posix_time::time_duration>(zeroTime, -remaining);
+}
+
+boost::posix_time::time_duration TimeFrame::getElapsedTime() const
+{
+	return boost::posix_time::microsec_clock::local_time() - mStartTime;
+}
+
 }
