@@ -20,18 +20,28 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.//
 //
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef _WIN32
+//Necessary to tell the mouse events to go to this window
+#if (_WIN32_WINNT < 0x0501)
+#undef _WIN32_WINNT
+#define _WIN32_WINNT 0x0501
+#endif
+// Necessary to get the Window Handle of the window
+// OGRE created, so SDL can grab its input.
+#include "windows.h"
+#include "SDL_getenv.h"
+#endif
+
 #include "Input.h"
-// #include "../EmberOgre.h"
+#include "IWindowProvider.h"
 #include "InputCommandMapper.h"
+#include "EmberIcon.h"
 
-#include "main/Application.h"
-#ifndef WITHOUT_SCRAP
-#include "framework/scrap.h"
-#endif // WITHOUT_SCRAP
 #include "IInputAdapter.h"
 
 #include "services/config/ConfigListenerContainer.h"
@@ -39,8 +49,33 @@
 #include "framework/Tokeniser.h"
 #include "framework/ConsoleBackend.h"
 #include "framework/LoggingInstance.h"
+#include "framework/MainLoopController.h"
 
+#ifdef _WIN32
+#include "platform/platform_windows.h"
+#endif
+
+#ifndef WITHOUT_SCRAP
+#include "framework/scrap.h"
+#endif // WITHOUT_SCRAP
+#ifdef _MSC_VER
+#include <SDL.h>
+#include <SDL_syswm.h>
 #include <SDL_keyboard.h>
+#else
+#include <SDL/SDL.h>
+#include <SDL/SDL_syswm.h>
+#include <SDL/SDL_keyboard.h>
+#endif
+
+#ifndef __APPLE__
+#ifndef _WIN32
+#include <X11/Xlib.h>
+#endif
+#endif
+
+#include <sstream>
+#include <algorithm>
 
 template<> Ember::Input* Ember::Singleton<Ember::Input>::ms_Singleton = 0;
 
@@ -51,7 +86,7 @@ const std::string Input::BINDCOMMAND("bind");
 const std::string Input::UNBINDCOMMAND("unbind");
 
 Input::Input() :
-		mCurrentInputMode(IM_GUI), mMouseState(0), mTimeSinceLastRightMouseClick(0), mSuppressForCurrentEvent(false), mMovementModeEnabled(false), mConfigListenerContainer(new ConfigListenerContainer()), mMouseGrabbingRequested(false)
+		ToggleFullscreen(0), mCurrentInputMode(IM_GUI), mMouseState(0), mTimeSinceLastRightMouseClick(0), mSuppressForCurrentEvent(false), mMovementModeEnabled(false), mConfigListenerContainer(new ConfigListenerContainer()), mMouseGrabbingRequested(false), mMouseGrab(false), mMainLoopController(0), mWindowProvider(nullptr), mScreenWidth(0), mScreenHeight(0), mIconSurface(0), mMainVideoSurface(0), mInvertMouse(1), mHandleOpenGL(false)
 {
 	mMousePosition.xPixelPosition = 0;
 	mMousePosition.yPixelPosition = 0;
@@ -77,38 +112,270 @@ Input::Input() :
 	mNonCharKeys.insert(SDLK_RETURN);
 	mNonCharKeys.insert(SDLK_DELETE);
 
+	//If building WebEmber on Linux we shouldn't initialize SDL here, as that's done separately in another thread.
+#if !defined(BUILD_WEBEMBER) || defined(_WIN32) || defined(__APPLE__)
+	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE);
+#endif
 	mLastTick = SDL_GetTicks();
 
+	//this is a failsafe which guarantees that SDL is correctly shut down (returning the screen to correct resolution, releasing mouse etc.) if there's a crash.
+	atexit(SDL_Quit);
+
 }
 
 Input::~Input()
 {
 	delete mConfigListenerContainer;
+	shutdownInteraction();
 }
 
-void Input::initialize(int width, int height)
+std::string Input::createWindow(unsigned int width, unsigned int height, bool fullscreen, bool resizable, bool centered, bool handleOpenGL)
 {
+	mHandleOpenGL = handleOpenGL;
+	int flags = SDL_HWSURFACE;
+
+	if (resizable) {
+		flags |= SDL_RESIZABLE;
+	}
+
+	if (mHandleOpenGL) {
+		flags |= SDL_OPENGL;
+	}
+
+	if (fullscreen) {
+		flags |= SDL_FULLSCREEN;
+	}
+
+	//On Windows we'll use another method to center the window once it has been created.
+	//In addition, if the environment variable is set on windows, it will never be cleared.
+	//As a result, every time the window is resized it will be adjusted to be centered.
+#ifndef _WIN32
+	if (centered) {
+		//Some versions of SDL_putenv accepts only a non-const char array.
+		char centered[] = "SDL_VIDEO_CENTERED=center";
+		SDL_putenv(centered);
+	}
+#endif
+
+	createIcon();
+	mMainVideoSurface = SDL_SetVideoMode(width, height, 0, flags); // create an SDL window
+
+	SDL_SysWMinfo info;
+	SDL_VERSION(&info.version);
+
+	SDL_GetWMInfo(&info);
+
+#ifdef _WIN32
+	if (centered) {
+		HWND hwnd = info.window;
+		HMONITOR hMonitor = MonitorFromWindow(hwnd, MONITOR_DEFAULTTONEAREST);
+		if (hMonitor) {
+
+			MONITORINFO mon;
+			memset(&mon, 0, sizeof(MONITORINFO));
+			mon.cbSize = sizeof(MONITORINFO);
+			GetMonitorInfo(hMonitor, &mon);
+
+			RECT r;
+			GetWindowRect(hwnd, &r);
+
+			int winWidth = r.right - r.left;
+			int winLeft = mon.rcWork.left + (mon.rcWork.right - mon.rcWork.left) / 2 - winWidth/2;
+
+			int winHeight = r.bottom - r.top;
+			int winTop = mon.rcWork.top + (mon.rcWork.bottom - mon.rcWork.top) / 2 - winHeight/2;
+			winTop = std::max(winTop, 0);
+
+			SetWindowPos(hwnd, 0, winLeft, winTop, 0, 0, SWP_NOSIZE);
+		}
+	}
+#endif	
+
+	SDL_WM_SetCaption("Ember", "ember");
+
 	SDL_ShowCursor(0);
 	SDL_EnableUNICODE(1);
 	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
 
+	std::stringstream ss;
+#ifdef __APPLE__
+	//On OSX we'll tell Ogre to use the current OpenGL context; thus we don't need to return the window id
+#elif _WIN32
+	ss << (size_t)info.window;
+#else
+	ss << info.info.x11.window;
+#endif
+
 	//must initialize the clipboard support
 #ifndef WITHOUT_SCRAP
 	if (init_scrap() < 0) {
 		S_LOG_FAILURE("Couldn't init clipboard: \n" << SDL_GetError());
 	}
 #endif // WITHOUT_SCRAP
-	ConsoleBackend::getSingletonPtr()->registerCommand(BINDCOMMAND, this);
-	ConsoleBackend::getSingletonPtr()->registerCommand(UNBINDCOMMAND, this);
+	ConsoleBackend& console = ConsoleBackend::getSingleton();
+	console.registerCommand(BINDCOMMAND, this);
+	console.registerCommand(UNBINDCOMMAND, this);
 
 	setGeometry(width, height);
 
+	if (!ToggleFullscreen) {
+		ToggleFullscreen = new ConsoleCommandWrapper("toggle_fullscreen", this, "Switch between windowed and full screen mode.");
+	}
+
+	return ss.str();
+}
+
+void Input::shutdownInteraction()
+{
+	mWindowProvider = 0;
+
+	if (mIconSurface) {
+		SDL_FreeSurface(mIconSurface);
+		mIconSurface = 0;
+	}
+
+	// according to http://www.libsdl.org/cgi/docwiki.cgi/SDL_SetVideoMode
+	// never free the surface returned from SDL_SetVideoMode
+	mMainVideoSurface = 0;
+
+	//Release the mouse for safety's sake.
+	SDL_WM_GrabInput(SDL_GRAB_OFF);
+	S_LOG_INFO("Shutting down SDL.");
+	SDL_Quit();
+	S_LOG_INFO("SDL shut down.");
+
+	ConsoleBackend& console = ConsoleBackend::getSingleton();
+	console.deregisterCommand(BINDCOMMAND);
+	console.deregisterCommand(UNBINDCOMMAND);
+	delete ToggleFullscreen;
+}
+
+void Input::createIcon()
+{
+#ifndef BUILD_WEBEMBER
+	//set the icon of the window
+	Uint32 rmask, gmask, bmask;
+
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+	rmask = 0xff000000;
+	gmask = 0x00ff0000;
+	bmask = 0x0000ff00;
+#else
+	rmask = 0x000000ff;
+	gmask = 0x0000ff00;
+	bmask = 0x00ff0000;
+#endif
+
+	//We'll use the emberIcon struct. This isn't needed on WIN32 or OSX as the icon is provided through different means.
+	mIconSurface = SDL_CreateRGBSurfaceFrom(emberIcon.pixel_data, 64, 64, 24, 64 * 3, rmask, gmask, bmask, 0);
+	if (mIconSurface) {
+		SDL_WM_SetIcon(mIconSurface, 0);
+	}
+
+#endif // !BUILD_WEBEMBER
+}
+
+void Input::attach(IWindowProvider* windowProvider)
+{
+
+	//The windowProvider should not be nullptr.
+	assert(windowProvider);
+	mWindowProvider = windowProvider;
+
+	char tmp[64];
+	// Set the SDL_WINDOWID environment variable
+	sprintf(tmp, "SDL_WINDOWID=%s", mWindowProvider->getWindowHandle().c_str());
+	putenv(tmp);
+
+	createIcon();
+	unsigned int width, height;
+	mWindowProvider->getWindowSize(width, height);
+	setGeometry(width, height);
+
+	SDL_ShowCursor(0);
+	SDL_EnableUNICODE(1);
+	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
+
+#ifdef _WIN32
+	if (width && height)
+	{
+		// if width = 0 and height = 0, the window is fullscreen
+
+		// This is necessary to allow the window to move
+		//  on WIN32 systems. Without this, the window resets
+		//  to the smallest possible size after moving.
+		SDL_SetVideoMode(width, height, 0, 0);// first 0: BitPerPixel,
+											  // second 0: flags (fullscreen/...)
+											  // neither are needed as Ogre sets these
+	}
+
+	static SDL_SysWMinfo pInfo;
+	SDL_VERSION(&pInfo.version);
+	SDL_GetWMInfo(&pInfo);
+
+	// Also, SDL keeps an internal record of the window size
+	//  and position. Because SDL does not own the window, it
+	//  missed the WM_WINDOWPOSCHANGED message and has no record of
+	//  either size or position. It defaults to {0, 0, 0, 0},
+	//  which is then used to trap the mouse "inside the
+	//  window". We have to fake a window-move to allow SDL
+	//  to catch up, after which we can safely grab input.
+	//  Note that the WM_WINDOWPOSCHANGED seems only to be sent if the
+	//  position of the window actually changed. Thus we have to first move
+	//  it one pixel to the right, and then back again.
+	RECT r;
+	GetWindowRect(pInfo.window, &r);
+	SetWindowPos(pInfo.window, 0, r.left + 1, r.top, 0, 0, SWP_NOSIZE);
+	SetWindowPos(pInfo.window, 0, r.left, r.top, 0, 0, SWP_NOSIZE);
+
+	RAWINPUTDEVICE Rid;
+	/* we're telling the window, we want it to report raw input events from mice (needed for SDL-1.3) */
+	Rid.usUsagePage = 0x01;
+	Rid.usUsage = 0x02;
+	Rid.dwFlags = RIDEV_INPUTSINK;
+	Rid.hwndTarget = (HWND)pInfo.window;
+	RegisterRawInputDevices(&Rid, 1, sizeof(Rid));
+#endif
+
+#ifndef __APPLE__
+#ifndef _WIN32
+
+	//The SDL_WINDOWID hack will result in no X events being caught on X11; we need to inform X11 that we want them anyway.
+	SDL_SysWMinfo info;
+	XSetWindowAttributes attributes;
+
+	SDL_VERSION(&info.version);
+	// this is important!
+	SDL_GetWMInfo(&info);
+
+	info.info.x11.lock_func();
+
+	attributes.event_mask = KeyPressMask | KeyReleaseMask | PointerMotionMask | ButtonPressMask | ButtonReleaseMask | EnterWindowMask | LeaveWindowMask;
+	XChangeWindowAttributes(info.info.x11.display, info.info.x11.window, CWEventMask, &attributes);
+	info.info.x11.unlock_func();
+
+#endif
+#endif
+	//must initialize the clipboard support
+#ifndef WITHOUT_SCRAP
+	if (init_scrap() < 0) {
+		S_LOG_FAILURE("Couldn't init clipboard: \n" << SDL_GetError());
+	}
+#endif // WITHOUT_SCRAP
+	ConsoleBackend& console = ConsoleBackend::getSingleton();
+	console.registerCommand(BINDCOMMAND, this);
+	console.registerCommand(UNBINDCOMMAND, this);
+
 }
 
 void Input::setGeometry(int width, int height)
 {
 	mScreenWidth = width;
 	mScreenHeight = height;
+	if (mMainVideoSurface->h != height || mMainVideoSurface->w != width) {
+		mMainVideoSurface = SDL_SetVideoMode(width, height, mMainVideoSurface->format->BitsPerPixel, mMainVideoSurface->flags);
+		EventSizeChanged.emit(width, height);
+	}
 }
 
 void Input::runCommand(const std::string &command, const std::string &args)
@@ -144,6 +411,8 @@ void Input::runCommand(const std::string &command, const std::string &args)
 				I->second->unbindCommand(key);
 			}
 		}
+	} else if (ToggleFullscreen && *ToggleFullscreen == command) {
+		setFullscreen((mMainVideoSurface->flags & SDL_FULLSCREEN) == 0);
 	}
 }
 
@@ -162,11 +431,23 @@ void Input::setMovementModeEnabled(bool value)
 	mMovementModeEnabled = value;
 }
 
-void Input::writeToClipboard(const std::string& text)
+void Input::writeToClipboard(char* text, size_t length)
 {
 #ifndef WITHOUT_SCRAP
-	char basicString[text.length() + 1];strcpy(basicString, text.c_str());
-	put_scrap(T('T','E','X','T'), text.length(), basicString);
+	put_scrap(T('T','E','X','T'), length, text);
+#endif // WITHOUT_SCRAP
+}
+
+void Input::pasteFromClipboard(char*& text, size_t& length)
+{
+#ifndef WITHOUT_SCRAP
+	static char *scrap = 0;
+	int scraplen;
+
+	get_scrap(T('T','E','X','T'), &scraplen, &scrap);
+
+	length = scraplen;
+	text = scrap;
 #endif // WITHOUT_SCRAP
 }
 
@@ -182,24 +463,21 @@ void Input::deregisterCommandMapper(InputCommandMapper* mapper)
 
 bool Input::isApplicationVisible()
 {
+	if (mWindowProvider) {
+		return mWindowProvider->isWindowVisible();
+	}
 	return SDL_GetAppState() & SDL_APPACTIVE;
 }
 
+bool Input::isApplicationFocused()
+{
+	return SDL_GetAppState() & SDL_APPMOUSEFOCUS;
+}
+
 void Input::startInteraction()
 {
-	//SDL 1.3 allow for easy raising of the window. Alas we're using 1.2
-//#ifdef SDL_VIDEO_DRIVER_X11
-//	SDL_SysWMinfo info;
-//	SDL_VERSION(&info.version);
-//
-//	if (SDL_GetWMInfo(&info)) {
-//		SDL_Window* window = info.info.x11.window;
-//		if (window) {
-//			SDL_RaiseWindow(SDL_Window* window);
-//		}
-//	}
-//#endif
 	mConfigListenerContainer->registerConfigListenerWithDefaults("input", "catchmouse", sigc::mem_fun(*this, &Input::Config_CatchMouse), true);
+	mConfigListenerContainer->registerConfigListenerWithDefaults("input", "invertcamera", sigc::mem_fun(*this, &Input::Config_InvertCamera), true);
 }
 
 void Input::processInput()
@@ -209,6 +487,10 @@ void Input::processInput()
 	mLastTick = ticks;
 	pollMouse(secondsSinceLast);
 	pollEvents(secondsSinceLast);
+	if (mWindowProvider) {
+		mWindowProvider->processInput();
+	}
+	SDL_GL_SwapBuffers();
 }
 
 void Input::pollMouse(float secondsSinceLast)
@@ -233,10 +515,10 @@ void Input::pollMouse(float secondsSinceLast)
 			MouseMotion motion;
 			motion.xPosition = mouseX;
 			motion.yPosition = mouseY;
-			motion.xRelativeMovement = diffX;
-			motion.yRelativeMovement = diffY;
-			motion.xRelativeMovementInPixels = mMousePosition.xPixelPosition - mouseX;
-			motion.yRelativeMovementInPixels = mMousePosition.yPixelPosition - mouseY;
+			motion.xRelativeMovement = diffX * mInvertMouse;
+			motion.yRelativeMovement = diffY * mInvertMouse;
+			motion.xRelativeMovementInPixels = (mMousePosition.xPixelPosition - mouseX) * mInvertMouse;
+			motion.yRelativeMovementInPixels = (mMousePosition.yPixelPosition - mouseY) * mInvertMouse;
 			motion.timeSinceLastMovement = secondsSinceLast;
 
 			EventMouseMoved.emit(motion, mCurrentInputMode);
@@ -282,7 +564,9 @@ void Input::pollEvents(float secondsSinceLast)
 			keyChanged(event.key);
 			break;
 		case SDL_QUIT:
-			Application::getSingleton().requestQuit();
+			if (mMainLoopController) {
+				mMainLoopController->requestQuit();
+			}
 			break;
 		case SDL_MOUSEBUTTONDOWN:
 		case SDL_MOUSEBUTTONUP:
@@ -377,43 +661,34 @@ void Input::pollEvents(float secondsSinceLast)
 		case SDL_ACTIVEEVENT:
 			if (event.active.state & SDL_APPACTIVE) {
 				EventWindowActive.emit(event.active.gain);
+//On Windows we get a corrupted screen if we just switch to non-fullscreen here.
+#ifndef _WIN32
+				lostFocus();
+#endif
 			}
 			break;
+
+		case SDL_VIDEORESIZE:
+			setGeometry(event.resize.w, event.resize.h);
+			break;
 		}
 	}
 }
 
-void Input::pasteFromClipboard()
-{
-#ifndef WITHOUT_SCRAP
-	static char *scrap = 0;
-	int scraplen;
-
-	get_scrap(T('T','E','X','T'), &scraplen, &scrap);
-	for (int i = 0; i < scraplen; ++i) {
-		for (IInputAdapterStore::const_iterator I = mAdapters.begin(); I != mAdapters.end();) {
-			IInputAdapter* adapter = *I;
-			++I;
-			if (!(adapter)->injectChar(scrap[i]))
-				break;
-		}
-	}
-#endif // WITHOUT_SCRAP
-}
-
 void Input::keyChanged(const SDL_KeyboardEvent &keyEvent)
 {
 	//catch paste key presses
 
 	//check for paste actions
-	if (((keyEvent.keysym.mod & KMOD_CTRL) || (keyEvent.keysym.mod & KMOD_LCTRL) || (keyEvent.keysym.mod & KMOD_RCTRL)) && (keyEvent.keysym.sym == SDLK_v)) {
+//On windows the OS will handle alt-tab independently, so we don't need to check here
+#ifndef _WIN32
+	if ((keyEvent.keysym.mod & KMOD_LALT) && (keyEvent.keysym.sym == SDLK_TAB)) {
 		if (keyEvent.type == SDL_KEYDOWN) {
-			pasteFromClipboard();
-		}
-	} else if ((keyEvent.keysym.mod & KMOD_LALT) && (keyEvent.keysym.sym == SDLK_TAB)) {
-		if (keyEvent.type == SDL_KEYDOWN) {
-			EventAltTab.emit();
+			lostFocus();
 		}
+#else
+		if (false) {
+#endif
 	} else {
 		if (keyEvent.type == SDL_KEYDOWN) {
 			mKeysPressed.insert(keyEvent.keysym.sym);
@@ -426,7 +701,7 @@ void Input::keyChanged(const SDL_KeyboardEvent &keyEvent)
 
 }
 
-const bool Input::isKeyDown(const SDLKey &key) const
+bool Input::isKeyDown(const SDLKey &key) const
 {
 	return mKeysPressed.find(key) != mKeysPressed.end();
 }
@@ -480,6 +755,7 @@ void Input::keyReleased(const SDL_KeyboardEvent &keyEvent)
 
 void Input::setInputMode(InputMode mode)
 {
+	setMouseGrab(mode == IM_MOVEMENT);
 	mCurrentInputMode = mode;
 	EventChangedInputMode.emit(mode);
 }
@@ -515,17 +791,21 @@ const MousePosition& Input::getMousePosition() const
 	return mMousePosition;
 }
 
-void Input::sleep(unsigned int milliseconds)
+void Input::setMainLoopController(MainLoopController* mainLoopController)
 {
-	SDL_Delay(milliseconds);
+	mMainLoopController = mainLoopController;
 }
 
+MainLoopController* Input::getMainLoopController() const
+{
+	return mMainLoopController;
+}
 
 void Input::Config_CatchMouse(const std::string& section, const std::string& key, varconf::Variable& variable)
 {
 	try {
 		if (variable.is_bool()) {
-			bool enabled = static_cast<bool> (variable);
+			bool enabled = static_cast<bool>(variable);
 			if (enabled) {
 				mMouseGrabbingRequested = true;
 			} else {
@@ -537,12 +817,55 @@ void Input::Config_CatchMouse(const std::string& section, const std::string& key
 	}
 }
 
-void Input::setMouseGrab(bool enabled) {
-	S_LOG_VERBOSE("Setting mouse catching to " << (enabled ? "enabled": "disabled"));
-	SDL_WM_GrabInput(enabled ? SDL_GRAB_ON : SDL_GRAB_OFF);
-	if (enabled) {
-		mMouseGrabbingRequested = false;
+void Input::Config_InvertCamera(const std::string& section, const std::string& key, varconf::Variable& variable)
+{
+	if (variable.is_bool()) {
+		mInvertMouse = static_cast<bool>(variable) ? -1 : 1;
 	}
 }
+
+void Input::setFullscreen(bool enabled)
+{
+	int newFlags = mMainVideoSurface->flags;
+
+	if (enabled) {
+		newFlags |= SDL_FULLSCREEN;
+	} else {
+		newFlags &= ~SDL_FULLSCREEN;
+	}
+	mMainVideoSurface = SDL_SetVideoMode(mMainVideoSurface->w, mMainVideoSurface->h, mMainVideoSurface->format->BitsPerPixel, newFlags);
+
+}
+
+bool Input::hasWindow() const
+{
+	return mMainVideoSurface != 0;
+}
+
+void Input::lostFocus()
+{
+	setInputMode(Input::IM_GUI);
+	setMouseGrab(false);
+	setFullscreen(false);
+//	EventWindowFocusChange.emit();
+}
+
+void Input::setMouseGrab(bool enabled)
+{
+	if (!enabled) {
+		SDL_WM_GrabInput(SDL_GRAB_OFF);
+		mMouseGrabbingRequested = false;
+	} else {
+		Uint8 appState = SDL_GetAppState();
+		if (enabled && !(appState & SDL_APPMOUSEFOCUS)) {
+			mMouseGrabbingRequested = true;
+		} else {
+			S_LOG_VERBOSE("Setting mouse catching to " << (enabled ? "enabled": "disabled"));
+			SDL_WM_GrabInput(enabled ? SDL_GRAB_ON : SDL_GRAB_OFF);
+			mMouseGrabbingRequested = false;
+		}
+	}
+}
+
 }
 
